@@ -1,7 +1,7 @@
 #ifndef GPS_H
 #define GPS_H
 
-#include <QObject>
+#include <QtCore>
 
 #ifdef Q_WS_MAEMO_5
 extern "C" {
@@ -10,7 +10,6 @@ extern "C" {
 }
 #else
 #include <QtNetwork>
-#include <QTimer>
 #endif // Q_WS_MAEMO_5
 
 class GpsMaemo5 : public QObject {
@@ -24,30 +23,36 @@ public:
     double get_lat() { return latitude; }
     double get_lon() { return longitude; }
     double get_acc() { return accuracy; }
+
+public slots:
+    void refresh();
+    void stop();
+    void forcestop();
+    int config(int i, int w, bool g);
+
+private slots:
 #ifdef Q_WS_MAEMO_5
     friend void GpsMaemo5_changed(LocationGPSDevice *device, GpsMaemo5 *gps);
-#endif // Q_WS_MAEMO_5
-
-#ifdef Q_WS_MAEMO_5
 #else
-private slots:
-    void finishedreply(QNetworkReply *r);
-    void gloc();
+    void gloc_reply(QNetworkReply *r);
 #endif // Q_WS_MAEMO_5
 
 private:
     double latitude;
     double longitude;
     double accuracy;
+    int interval;
+    int wait;
+    bool usegps;
+    bool emitfix;
+    bool stopgps;
 #ifdef Q_WS_MAEMO_5
-    GMainLoop *loop;
     LocationGPSDControl *control;
     LocationGPSDevice *device;
 #else
     QUrl urlloc;
     QNetworkAccessManager *worker;
 #endif // Q_WS_MAEMO_5
-
 };
 
 #ifdef Q_WS_MAEMO_5
